local LDB = LibStub:GetLibrary("LibDataBroker-1.1")
local LQT = LibStub("LibQTip-1.0")
local abacus = LibStub("LibAbacus-3.0")
local crayon = LibStub("LibCrayon-3.0")
local dataobj = LDB:NewDataObject("Broker_kAddonMsg", {
	type = "data source",
	text = "kAddonMsg",
	icon = "Interface\\AddOns\\Broker_kAddonMsg\\icon.tga",
})
local icon = LibStub("LibDBIcon-1.0")
local time = time
local pairs = pairs
local ipairs = ipairs
local table_sort = table.sort
local BNSendGameData = BNSendGameData
local timerSched = {}
local purged = false

Broker_kAddonMsg = LibStub("AceAddon-3.0"):NewAddon("Broker_kAddonMsg", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")
local self, Broker_kAddonMsg = Broker_kAddonMsg, Broker_kAddonMsg
local db
local tooltip
local defaults = {
	profile = {
		minimap = {
			hide = false,
		},
		filter = {
			realm = {},
			char = {},
		},
	},
	global = {},
}

local function GetOptions(uiType, uiName)
	local options = {
		type = "group",
		name = GetAddOnMetadata("Broker_kAddonMsg", "Title"),
		args = {
			desc = {
				type = "description",
				order = 0,
				name = GetAddOnMetadata("Broker_kAddonMsg", "Notes"),
			},
			minimap = {
				name = L["Minimap Icon"],
				desc = L["Toggle minimap icon"],
				type = "toggle",
				order = 10,
				get = function() return not db.minimap.hide end,
				set = function()
					db.minimap.hide = not db.minimap.hide
					if db.minimap.hide then
						icon:Hide("Broker_kAddonMsg")
					else
						icon:Show("Broker_kAddonMsg")
					end
				end,
			},
		},
	}
	return options
end

function Broker_kAddonMsg:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("Broker_kAddonMsgDB", defaults, true)
	db = self.db.profile

	-- Minimap Icon
	icon:Register("Broker_kAddonMsg", dataobj, db.minimap)

	-- Options
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Broker_kAddonMsg", GetOptions)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Broker_kAddonMsg", GetAddOnMetadata("Broker_kAddonMsg", "Title"))
end

function Broker_kAddonMsg:OnEnable()
	self:Hook('BNSendGameData', 'MessageSent', true)
	self:Hook('SendAddonMessage', 'MessageSent', true)	
	self:RegisterEvent('CHAT_MSG_ADDON', 'MessageReceived')
	self:RegisterEvent('BN_CHAT_MSG_ADDON', 'MessageReceived')
end

function Broker_kAddonMsg:MessageSent(...)
	print('Broker_kAddonMsg:MessageSent', ...)
end

function Broker_kAddonMsg:MessageReceived(...)
	print('Broker_kAddonMsg:MessageReceived', ...)
end

function Broker_kAddonMsg:DrawTooltip()
	-- Don't show if we're in combat.
	if InCombatLockdown() then
		return
	end

	tooltip:Hide()
	tooltip:Clear()

	local myFont
	if not Broker_kAddonMsg_Tooltip_Font then
		myFont = CreateFont("Broker_kAddonMsg_Tooltip_Font")
		local filename, size, flags = tooltip:GetFont():GetFont()
		myFont:SetFont(filename, size, flags)
		myFont:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
	else
		myFont = Broker_kAddonMsg_Tooltip_Font
	end
	tooltip:SetFont(myFont)

	-- Header
	tooltip:AddHeader(nil, GetAddOnMetadata("Broker_kAddonMsg", "Title"), nil)
	tooltip:AddLine(" ")
	tooltip:AddHeader("First", "Second", "Third")
	
	tooltip:AddLine("Testing", "another", 1234)

	--local playedTimeText = abacus:FormatDurationCondensed(playedTime, true, true)
	--abacus:FormatDurationExtended(totalTimePlayed, true, true)

	tooltip:UpdateScrolling()
	tooltip:Show()
end

-- LDB functions
function dataobj:OnEnter()
	if not LQT:IsAcquired("Broker_kAddonMsg") then
		tooltip = LQT:Acquire("Broker_kAddonMsgTip",
			-- Columns
			3,
			-- Alignments
			-- Realm, TimePlayed, LastPlayed, TimeToRest, CurrentXP, RestXP, Zone
			"LEFT", "CENTER", "RIGHT"
		)
	end
	tooltip:Clear()
	tooltip:SmartAnchorTo(self)
	tooltip:SetAutoHideDelay(0.25, self)
	tooltip:SetScale(1)

	Broker_kAddonMsg:DrawTooltip()
end

-- Handled by the AutoHide
--function dataobj:OnLeave()
--	LQT:Release(tooltip)
--	tooltip = nil
--end

function dataobj:OnClick(button)
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata("Broker_kAddonMsg", "Title"))
	end
end
